"""
Created on 2 April 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

https://en.wikipedia.org/wiki/List_of_HTTP_status_codes
"""


# TODO: remove, and replace with httpstatus package
# --------------------------------------------------------------------------------------------------------------------

class HTTPStatus(object):
    """
    classdocs
    """

    OK =                200
    CREATED =           201
    NO_CONTENT =        204

    BAD_REQUEST =       400
    FORBIDDEN =         403
    NOT_FOUND =         404
